#!/usr/local/bin/perl
#
# Online Journal program by Sergey Zozulevich, zuzusoft@usa.net
# http://kompashka.hypermart.net/download/
# Version 1.1, released June 15, 1999
#
# Original idea by Julie Tran, jul@nucleus.com
# http://www.nucleus.com/~jul/ 
#
# We use cgi-lib.pl by Steven E. Brenner, cgi-lib@pobox.com
# http://cgi-lib.stanford.edu/cgi-lib/
#

### INITIALIZATION - BEGIN ###

require("cgi-lib.pl");

# Find out the script's URL (will be used for navigation links):
$journal_url = MyBaseUrl(); ### See cgi-lib.pl

# Find out the full and complete path to the script's directory
# (necessary to be able to open data files):
@d_fs = split (/\//, $ENV{'SCRIPT_FILENAME'});
$k_d = grep ((!( /\// )), @d_fs);
$script_path = @d_fs[0];
for ($k = 1; $k < $k_d - 1; $k++) {
  $script_path = $script_path . "/" . @d_fs[$k];
} # end for

# Read any passed-in input, and put it into a hash named "in":
&ReadParse; ### See cgi-lib.pl

# Try to execute the default configuration file:
if (! do "defaultconfig.pl") {
  print "Content-type: text/html\n\n";
  print <<EOF;
<HTML>
<BODY>
Error : 'defaultconfig.pl' file execution failed!<BR>
Check 'defaultconfig.pl' file for syntax errors<BR> 
(try : perl -d defaultconfig.pl).
</BODY>
</HTML>
EOF
  exit(0);
} # end if - default configuration execution

# Check whether we are opening the default journal or another one:
  $folder_name = $in{'folder'} if ($in{'folder'} ne "");
  $journal_dir = "$script_path/$folder_name";
  $journal_url = "$journal_url?folder=$folder_name";

$headersrc = "$journal_dir/$header_name";
$footersrc = "$journal_dir/$footer_name";

# Now that we know this journal's folder, try to execute the individual
# configuration file, initializing variables:
if (! do "$folder_name/$jconfig_name") {
  print "Content-type: text/html\n\n";
  print <<EOF;
<HTML>
<BODY>
Error : '$folder_name/$jconfig_name' file execution failed!<BR>
Check '$folder_name/$jconfig_name' file for syntax errors<BR>
(try : perl -d $journal_dir/$jconfig_name).
</BODY>
</HTML>
EOF
  exit(0);
} # end if - individual configuration execution

### initialization - end ###

### EXECUTION - BEGIN ###

# Get Journal entries filenames into array
opendir (JDIR, "$journal_dir/"); 
@jchapters = readdir(JDIR);
closedir (JDIR);
@jchapters = grep(/\.txt/, @jchapters);
$total_chapters = grep(/\.txt/, @jchapters);
foreach $jchapter(@jchapters) {
  ($chap_num, $trash) = split (/\./, $jchapter);
  push (@chap_nums, $chap_num);
} # end foreach
@sorted_chapters = sort(@chap_nums);

# Starting the output. Print the magic header for the user's browser:
print "Content-type: text/html\n\n";

# Read and print out the header file
&Print_File_Out($headersrc);

# Main analysis - 4 cases, depending on the 'next' parameter value
# 'next' should be either 'outline', or 'last', or an integer

if ($in{'next'} eq "") {
  # (1) No parameter given: Check the default behaviour 

  # Default behaviour analysis - 3 cases

  if ($default_startup eq "0") {
  # (a) Print the journal outline
  
    &Go_Print_Outline;
  
  } elsif ($default_startup eq "1") {
  # (b) Print the first entry
    $enumb = 0;
    $prev_num = 0;
    $next_num = 2;
    
    &Print_A_Chapter;
    &Links_Next_Prev;
    
  } else {
  # (c) Print the last entry
    $enumb = $total_chapters - 1;
    $prev_num = $total_chapters - 1;
    $next_num = $total_chapters + 1;
    
    &Print_A_Chapter;
    &Links_Next_Prev;
    
  } # end if - default behaviour analysis
  
} elsif ($in{'next'} eq "last") {
  # (2) Print the last entry
  $enumb = $total_chapters - 1;
  $prev_num = $total_chapters - 1;
  $next_num = $total_chapters + 1;
  
  &Print_A_Chapter;
  &Links_Next_Prev;

} elsif ($in{'next'} eq "outline") {
  # (3) Print the journal outline - list entries
  
  &Go_Print_Outline;
  
} else {
  # (4) Print the indicated entry (chapter)
  
  $enumb = $in{'next'} - 1;
  $prev_num = $in{'next'} - 1;
  $next_num = $in{'next'} + 1;
  
  &Print_A_Chapter;
  &Links_Next_Prev;
  
} # end if - main analysis

# Read and print out the footer file
&Print_File_Out($footersrc);

exit(0);

### execution - end ###

### SUBROUTINES ###

sub Print_File_Out {
# Read and print out the specified file (used to print header / footer)

local ($file_to_print) = @_;
open (FOOT, $file_to_print);
@DATA=<FOOT>;
close (FOOT);
foreach $item (@DATA) {
  print "$item";
} # end foreach

} # end sub


sub Links_Next_Prev {
# Print links: first, prev/next, last, contents
# determined by $prev_num and $next_num variables
print "$navbar_format\n";

# Print FIRST and PREVIOUS links
if ($prev_num == 0) {
  print " $left_link_bracket$first_link_text$right_link_bracket" if ($first_link_text ne "");
  print " $left_link_bracket$prev_link_text$right_link_bracket" if ($prev_link_text ne "");
} else {
  print " $left_link_bracket<a href=\"$journal_url\&next=1\">$first_link_text</a>$right_link_bracket" if ($first_link_text ne "");
  print " $left_link_bracket<a href=\"$journal_url\&next=$prev_num\">$prev_link_text</a>$right_link_bracket" if ($prev_link_text ne "");
} # end if

# Print NEXT and LAST links
if ($next_num == ($total_chapters + 1)) {
  print " $left_link_bracket$next_link_text$right_link_bracket" if ($next_link_text ne "");
  print " $left_link_bracket$last_link_text$right_link_bracket" if ($last_link_text ne "");
} else {
  print " $left_link_bracket<a href=\"$journal_url\&next=$next_num\">$next_link_text</a>$right_link_bracket" if ($next_link_text ne "");
  print " $left_link_bracket<a href=\"$journal_url\&next=$total_chapters\">$last_link_text</a>$right_link_bracket" if ($last_link_text ne "");
} # end if

# Print CONTENTS and HOME links
print " $left_link_bracket<a href=\"$journal_url\&next=outline\">$contents_link_text</a>$right_link_bracket" if ($contents_link_text ne "");
print " $left_link_bracket<a href=\"$home_link_url\">$home_link_text</a>$right_link_bracket" if ($home_link_text ne "");

print "$left_link_bracket" if (!$right_link_bracket);

print "$navbar_format_close<BR>\n";

} # end sub


sub Print_A_Chapter {
# Print the entry (chapter) indicated by $enumb variable

open (JENTRY, "$journal_dir/$sorted_chapters[$enumb].txt");
$ctitle = <JENTRY>;
chomp($ctitle);
print "$title_format$ctitle$title_format_close<BR>\n";
print "$text_format";
while (<JENTRY>) {
  print $_;
  print "<BR>" if ($lf_to_br);
} # end while
close (JENTRY);
print "$text_format_close<BR>\n";

} # end sub


sub Print_Outline_Reverse {
# Print the journal outline - list entries in reverse order

print "$title_format $contents_link_text $title_format_close<BR><BR>";
for ($i = $total_chapters; $i > 0; $i--) {
  open (JENTRY, "$journal_dir/$sorted_chapters[($i-1)].txt");
  $ctitle = <JENTRY>;
  chomp($ctitle);
  print "<a href=\"$journal_url\&next=$i\">$chapter_prefix$sorted_chapters[($i-1)]</a> $ctitle<BR>";
} # end for

} # end sub


sub Print_Outline_Direct {
# Print the journal outline - list entries in direct order

print "$title_format $contents_link_text $title_format_close<BR><BR>";
for ($i = 1; $i <= $total_chapters; $i++) {
  open (JENTRY, "$journal_dir/$sorted_chapters[($i-1)].txt");
  $ctitle = <JENTRY>;
  chomp($ctitle);
  print "<a href=\"$journal_url\&next=$i\">$chapter_prefix$sorted_chapters[($i-1)]</a> $ctitle<BR>";
} # end for

} # end sub


sub Go_Print_Outline {
# Print the journal outline - list entries in reverse or direct order

if ($reverse_order) {
  &Print_Outline_Reverse;
} else {
  &Print_Outline_Direct;
} # end if

} # end sub
